from wxPython.wx import *
from twisted.spread import pb
from twisted.internet import reactor
from twisted.cred import credentials
from twisted.python import failure

def getDeferredResult(d, timeout=None):
    """Waits for a Deferred to arrive, then returns or throws an exception, based on the result.
    """
    from twisted.internet import reactor
    if timeout is not None:
        d.setTimeout(timeout)
    resultSet = []
    d.addCallbacks(resultSet.append, resultSet.append)
    while not resultSet:
        reactor.iterate()
    result = resultSet[0]

    if isinstance(result, failure.Failure):
        print "deferredResult::raise"
        raise result
    else:
        return result
        

class ServerCanCallThisObject(pb.Referenceable):
  def remote_write(self, stuff):
    wxGetApp().mainWindow.textEcho.WriteText(stuff)


class wxMyFrame(wxFrame):
  def __init__(self, parent):
    wxFrame.__init__(self, parent, -1, "MyFrame", size=(400,400),
                         style=wxDEFAULT_FRAME_STYLE | wxNO_FULL_REPAINT_ON_RESIZE)
    panel = wxPanel(self, -1)

    buttonClose = wxButton(panel, 1003, "Close")
    buttonClose.SetPosition(wxPoint(15, 15))

    buttonEcho = wxButton(panel, 1004, "Echo")
    buttonEcho.SetPosition(wxPoint(15, 45))
    
    buttonEcho = wxButton(panel, 1005, "Hello")
    buttonEcho.SetPosition(wxPoint(15, 75))    
    
    self.textEcho = wxTextCtrl(panel,1006)
    self.textEcho.SetPosition(wxPoint(15, 105))    

    EVT_BUTTON(self, 1003, self.OnCloseWindow)
    EVT_BUTTON(self, 1004, self.OnButtonEcho)
    EVT_BUTTON(self, 1005, self.OnButtonHello)
    EVT_CLOSE(self, self.OnCloseWindow)


  def OnCloseWindow(self, event):
    self.CloseWindow()

  def CloseWindow(self):   
    wxGetApp().factory.disconnect()  
    self.Destroy()

  def OnButtonEcho(self,event):
    ret = wxGetApp().echo("blob")
    self.textEcho.WriteText(ret)

  def OnButtonHello(self,event):
    wxGetApp().hello(ServerCanCallThisObject())
    

    

class PBFactory(pb.PBClientFactory):
    def __init__(self):
        pb.PBClientFactory.__init__(self)

    def clientConnectionFailed(self, connector, reason):
        print "clientConnectionFailed", reason
        pb.PBClientFactory.clientConnectionFailed(self,connector, reason)

    def clientConnectionMade(self, broker):
        print "connection made with broker", id(broker), broker
        pb.PBClientFactory.clientConnectionMade(self,broker)
        
    def clientConnectionLost(self, connector, reason):
        print "clientConnectionLost", reason
        pb.PBClientFactory.clientConnectionLost(self,connector, reason)
        wxGetApp().mainWindow.CloseWindow()
        

class wxMyApp(wxApp):
    def __init__(self):
        self.perspective = None
        self.viewpoint = None
        wxApp.__init__(self,0)

    def OnInit(self):
        result = 1

        # Start Twisted Code 
        self.factory = PBFactory()
        deferredPerspective = self.factory.login(credentials.UsernamePassword("blub", "123456"), "brains")
        reactor.connectTCP("localhost", 8080, self.factory)
        try:
          self.perspective = getDeferredResult(deferredPerspective)
          self.viewpoint = getDeferredResult(self.perspective.callRemote("getAView"))

          EVT_TIMER(self,999999,self.OnTimer) #the timer replaces reactor.run(), see doc-string in OnTimer(...)
          self.timer=wxTimer(self,999999)
          self.timer.Start(250,False)
        except:
          #propably no connection possible, so we don't need the timer
          result = 0

        # End Twisted Code
        
        # Start wxPython Code
        if result == 1:
          self.mainWindow = wxMyFrame(None)
          self.mainWindow.Show(1)
          self.SetTopWindow(self.mainWindow)
        # End wxPython Code
        return result
    
    def echo(self,x):
        return getDeferredResult(self.viewpoint.callRemote("echo",x))
  
    def hello(self,x):
        self.viewpoint.callRemote("hello",x)
        
    def run(self):
        self.MainLoop()
        
    def OnTimer(self,event):
        """
        http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/181780
        The reactor calls are copied from the select reactors mainloop.
        Note that the reactor.run() method is never called. This neverending loop 
        is emulated by the timer events. 
        Initialize your wxApp as usual by calling app.MainLoop() in your main module.
        You might have to play with the timer value (250 ms above) as well as with 
        the timeout value of the doIteration call. If you have only a few remote calls 
        you can increase the timer value - so the OnTimer event will be triggered less 
        often. This will increase the responsetime of your application. If you do heavy 
        networking via Twisted you might want to increase the doIteration timeout (maybe 0.002) 
        and probably decrease the timer value to somewhere around 150ms.
        Since the wx mainloop is running normally menus and modal dialogs won't block.
        """
        reactor.runUntilCurrent()
        reactor.doIteration(0)
        pass

def main():
    app = wxMyApp()
    app.run()
  
  
if __name__ == '__main__':
	main()