from twisted.cred.portal import IRealm
from twisted.cred import checkers 
from twisted.cred import portal 
from twisted.internet import reactor
from twisted.spread import pb

from threading import Timer



class MyPortal(portal.Portal):
    """
    This is the ineffable mystery at the core of login.  There is one 
    concrete implementation of Portal, and no interface - it does a very 
    simple task.  A Portal associates one (1) Realm with a collection of 
    CredentialChecker instances.  (More on those later.)
    The closest analogue of this in Old Cred was the Authorizer.
    If you are writing a protocol that needs to authenticate against 
    something, you will need a reference to a Portal, and to nothing else.  
    This has only 2 methods -
    	login(credentials, mind, *interfaces)
    The docstring is quite expansive (see twisted.cred.portal), but in 
    brief, this is what you call when you need to call in order to connect 
    a user to the system.  The result is a deferred which fires a tuple of:
    	- interface (which was one of the interfaces passed in the *interfaces tuple)
    	- an object that implements that interface (an Aspect of an Avatar)	- 
    logout, a 0-argument callable which disconnects the connection that was 
    established by this call to login
    	registerChecker(checker, *credentialInterfaces)
    which adds a CredentialChecker to the portal.
    """
    pass


class MyRealm:
    """
    A realm is an interface which connects your universe of "business 
    objects" to the authentication system.  This is similar to the Old Cred 
    "Service", but the name "Service" will be phased out when referring to 
    cred - another planned refactoring is to move 
    twisted.internet.app.ApplicationService into its own module and more 
    heavily emphasize its use in start-up and shut-down.
    IRealm is another one-method interface:
    	requestAvatar(avatarId, mind, *interfaces)
    This method will typically be called from 'Portal.login'.  The avatarId 
    is the one returned by a CredentialChecker.
    The important thing to realize about this method is that if it is being 
    called, _the user has already authenticated_.  Therefore, if possible, 
    the Realm should create a new user if one does not already exist 
    whenever possible.  Of course, sometimes this will be impossible 
    without more information, and that is the case that the interfaces 
    argument is for.
    Some protocols can only accept a fixed set of interfaces.  However, 
    others (most notably PB) can deal with more arbitrary data being 
    returned.  An online game may call Portal.login with:
         login(PBChallengeResponse(...), clientSideThingy,
               IServerPlayerInWorld, IUserChooser, ICharacterCreationThing)
    This specifies that the client would most prefer a player in the world, 
    but, failing that, a UserChooser (selection interface for a second step 
    to get a PlayerInWorld) or a CharacterCreationThing (interface to 
    create a new character in the world before returning it) would be 
    acceptable.
    Since requestAvatar should be called from a Deferred callback, it may 
    return a Deferred or a synchronous result.
    At the moment, there is only an interface for the Realm.  However, it 
    is expected that a utility class will be written in the near future to 
    facilitate log-in methods, a re-directing method similar to 
    "Perspective.attached" for Avatars, and at least somewhat automated 
    compositing of Realms.  However, in writing the code for cred and 
    guard, there has been no need for such a thing yet.
    """
    def __init__(self):
        self.p = MyPerspective()
    
    def requestAvatar(self, avatarId, mind, interface):
        """
        This object has the dubious distinction of appearing nowhere in the 
        code; in fact, very few things will ever touch an actual Avatar, and it 
        is not clear that it will have an interface.  However, _aspects_ of an 
        avatar are returned in deferreds from the above methods, which is to 
        say, things that implement particular interfaces which communicate with 
        various protocols that may access your Realm.
        This is (hopefully, obviously) similar to a 'Perspective'.  However, an 
        Avatar separates 2 concerns which were muddied previously - _access_ to 
        a realm and _storage of user data_ within a realm.  The avatar itself 
        stores the data, and the aspects interface to it.  The avatar itself 
        may of course implement its own aspects, but it is suggested in most 
        cases to register adapters that do this.  (Perspective Broker will keep 
        its name, because the Avatar Aspect that communicates with the remote 
        object protocol will still be called a Perspective.)
        """      
        self.p.loggedIn = 1
        return pb.IPerspective, self.p, self.p.logout

class MyPerspective(pb.Perspective):

    __implements__ = pb.IPerspective,

    def __init__(self):
        pass
    
    def perspective_getAView(self):
        return MyView()

    def logout(self):
        self.loggedOut = 1
        

class MyView(pb.Viewable):

    def view_check(self, user):
        return isinstance(user, MyPerspective)

    def view_echo(self, user, word):
        return word

    def view_hello(self, user, remoteObj):
        self.remoteObj = remoteObj
        #start a timer to show that we can do the remote call also async, or server triggered
        t = Timer(1.0, self.timer_hello)
        t.start() 
        
    def timer_hello(self):
        self.remoteObj.callRemote("write", "hallo")

    

checker = checkers.InMemoryUsernamePasswordDatabaseDontUse()
checker.addUser("blub", "123456")

port   =  8080
realm  =  MyRealm()
portal = MyPortal(realm)
portal.registerChecker(checker)
factory = pb.PBServerFactory(portal)

reactor.listenTCP(port, factory, interface="localhost")
print "running on ", port
reactor.run()
print "stopped"


